% Simplified Matlab Compass Simulation Code -- Scanning Pencil-beam case
% 
% For scanning pencil-beam scatterometer, incidence angle is essentially
% constant, but the azimuth angle of the antenna rotates 0 to 360.  At
% a given cross-track distance, the pairs of azimuth angles (one looking
% forward, the other after) observe the same point. When multiple
% incidence angle beams are available, some areas of the swath will have
% multiple pairs of azimuth angles.  This illustration code uses only a 
% single rotating beam at an incidence angle of 45 deg.
%
% For a particular incidence angle configuration (note: for CMOD5 there
% is only a single polarization), the azimuth angles are based on the
% cross-track distance.  The compass simulation considers a particular wind
% speed and considers wind directions from 0 to 360 deg ("around the
% compass").  Simulated backscatter observations are created using the 
% CMOD5 model function. Monte Carlo radiometric noise is added with a specified
% "radiometric resolution" (Kp) and the wind is estimated using a simple
% retrieval scheme based on minimum least-squares. Since there are
% multiple local minimia ("ambiguities") for the wind speed and direction
% estimates, the ambiguity that minimizes the difference between the estimate
% and the true wind is selected.  This is "ideal" point-wise ambiguity
% selection.  The error between the estimate and true wind is computed
% over multiple realizations of the noise to estimate the mean and RMS
% error for each true wind speed and direction and the results plotted.
% See Chapter 16 of (F. Ulaby and D.G. Long, Microwave Radar and 
% Radiometric Remote Sensing, ISBN: 978-0-472-11935-6, University of
% Michigan Press, Ann Arbor, Michigan, 2013.)
%
% The particular wind estimate code used is a simple (and inefficient) method
% based on a ridge search which intended to merely illustrate wind
% retrieval -- more sophisticated (and complicated) schemes are used for
% operation wind estimation.
%
% written by D.G. Long at BYU Dec 2011
%

clear

% set measurement geometry
r=1200;             % scan radius (km)
CRs=100:500:1200;   % consider multiple cross-track distances (km)
incidence=[45,45]'; % beam incidence angles

% set true wind speed
true_spd=10;   % m/s

% set of true wind directions to consider
true_dirs=0:10:350; % deg

% set noise level and realizations
Kp=0.05;
realizations=100; % number of noise realizations

% for each cross-track position
for icr=1:length(CRs)
  CR=CRs(icr);

  % compute azimuth angles 
  phif=asin(CR/r)*180/pi;
  phia=180-phif;

  azimuth=[phif,phia]';  % beam azimuth angles

  % predeclare output arrays
  sm=zeros(size(true_dirs));
  sr=zeros(size(true_dirs));
  dm=zeros(size(true_dirs));
  dr=zeros(size(true_dirs));
  
  % for each direction "around the compass" at a fixed speed and location
  for tdir=1:length(true_dirs)
    true_dir=true_dirs(tdir);
    fprintf('Swath Loc %d: True wind speed %d and direction %d\n',CR,true_spd,true_dir);

    % generate true noise-free sigma-0 values
    z=zeros(size(azimuth));
    arg=[true_spd+z,true_dir+z,azimuth,incidence];
    sigs=CMOD5(arg)';

    % initialize error statistics
    dmerr=0;
    dverr=0;
    smerr=0;
    sverr=0;
    icnt=0;
  
    % repeat simulation for multiple noise realizations
    for inoise=1:realizations
      % noise to sigma-0 measurements
      sigsn=sigs.*(1+Kp*rand(size(sigs)));
      
      % prepare argument to ravine search
      rarg=[azimuth,incidence,sigsn];

      % find ambiguities using ravine search
      [ambs, plt]=ravine_search_CMOD5(rarg);

      % direction error can be tricky to compute correctly
      % since is must be between +/- 180 deg
      % use a slick trick for the absolute value of the direction error
      de=180-abs(180-abs(mod(ambs(:,2)-true_dir,360)));
    
      % find closest ambiguity to true wind, defined to be the 
      % ambiguity closest in direction to the true wind
      [y,i]=min(de);
        
      % get the signed angle error for the selected ambiguitiy i
      derr=de(i);
      if 180-abs(180-abs(mod(ambs(:,2)-true_dir+de,360))) < 0.001
	derr=-derr;
      end
    
      % accumulate speed statistics
      serr=ambs(i,1)-true_spd;
      smerr=smerr+serr;
      sverr=sverr+serr^2;
      icnt=icnt+1;
      
      % accumulate the directional statistics
      dmerr=dmerr+derr;
      dverr=dverr+derr^2;
    end
  
    % complete computation of error statistics
    sm(tdir)=smerr/icnt;
    sr(tdir)=sqrt(sverr/icnt);
    dm(tdir)=dmerr/icnt;
    dr(tdir)=sqrt(dverr/icnt);
  end  

  % compute variance
  sv=sqrt(sr.^2-sm.^2);
  dv=sqrt(dr.^2-dm.^2);

  % plot compass simulation results
  figure(icr);clf;
  subplot(2,1,1)
  plot(true_dirs,sm,'r')
  hold on;
  plot(true_dirs,sr,'b')
  plot(true_dirs,sv,'g')
  hold off
  title(sprintf('Cross-Track %d km  speed err blue=RMS green=var red=mean',CR))
  xlabel('wind direction (deg)');
  ylabel('wind speed error (m/s)');

  subplot(2,1,2)
  plot(true_dirs,dm,'r')
  hold on;
  plot(true_dirs,dr,'b')
  plot(true_dirs,dv,'g')
  hold off
  title(sprintf('Cross-track %d km  direction err blue=RMS green=variance red=mean',CR))
  xlabel('wind direction (deg)');
  ylabel('wind direction error (deg)');

  % save graphic
  if icr==1
    print -dpng InnerSwath_CompassSim.png
  elseif icr==2
    print -dpng MidSwath_CompassSim.png
  else % icr==3
    print -dpng FarSwath_CompassSim.png
  end
  
end
